/*
uberflate compression library 0.1.1
    kzip+deflopt+defluff+deflopt wrapper
by Luigi Auriemma
e-mail: aluigi@autistici.org
web:    aluigi.org

  int uberflate(        // return the size of the compressed data or -1 if error
    void *input,        // decompressed buffer (input)
    int input_length,   // decompressed size (input size)
    void *output,       // destination buffer for the compressed data (output)
    int output_size,    // size of the output buffer (output size)
    int zlib            // 0 for deflate, 1 for zlib
  );

This function/wrapper allows to use the awesome compression performed by
the kzip [1] tool written by Ken Silverman plus the Huffman optimizations
of deflopt [2] and defluff [3] for removing additional bytes:

  kzip(/b128) + deflopt + defluff + deflopt = extreme deflate compression

[1] http://advsys.net/ken/utils.htm
[2] http://www.walbeehm.com/download/
[3] http://encode.ru/threads/1214-defluff-a-deflate-huffman-optimizer

The last "zlib" argument must be 0 for the deflate compression (RFC1591)
or 1 for the zlib format (RFC1590: header + deflate + crc).

The output buffer can be allocated using the following macro:

  outsz = UBERFLATE_MAXZIPLEN(insz);
  out   = malloc(outsz);

If "output" is NULL then the compressed output will be copied in the
input buffer:

  outsz = uberflate(in_out, insz, NULL, 0, 0);

Uberflate uses the internal function quickbms_tmpname available in
QuickBMS http://quickbms.aluigi.org src/utils.h, but I have implemented
a stand-alone version here too (feel free to enhance it you want).

The job of this function is:
- launching the executables silently from the %TEMP% folder
- using named pipes for avoiding the usage of the disk
- emulating the access to the input/output files (named pipes are streams)
- patching the original executables to make them compatible with named pipes
- providing a simple interface for input=>output memory compression

This function has been tested on Windows XP, 7 and 8.
The main target of this function are small files under 256 Mb to avoid to
waste time and memory.
*/ 



// original zlib maximum output size: ((n)+(((n)/1000)+1)+12)
// please check manually if after UBERFLATE_MAXZIPLEN n is smaller
// than your original n (integer overflow)
#define UBERFLATE_MAXZIPLEN(n)  ((n)+(((n)/10)+1)+256)



int uberflate(
    void *,
    int,
    void *,
    int,
    int);

